// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Event callback typedef
typedef Future<Null> OmnibusEventCallback<T>(T message);

/// Request callback typedef
typedef Future<OmnibusResponse> OmnibusRequestCallback<T>(T message);

/// Handler semantics
abstract class OmnibusHandler {
  /// Type (event or request channel)
  final OmnibusMessageType type;

  /// Target omnibusId
  final String omnibusId;

  /// Constructor
  OmnibusHandler(this.type, this.omnibusId);
}

abstract class OmnibusEventHandler extends OmnibusHandler {
  /// Constructor
  OmnibusEventHandler(String omnibusId)
      : super(OmnibusMessageType.event, omnibusId);

  /// Process incoming message.
  List<Future<Null>> process(OmnibusEvent data);
}

abstract class OmnibusRequestHandler extends OmnibusHandler {
  /// Constructor
  OmnibusRequestHandler(String omnibusId)
      : super(OmnibusMessageType.request, omnibusId);

  /// Process incoming message.
  List<Future<OmnibusResponse>> process(OmnibusRequest data);
}
